import javax.swing.*;
import java.io.*;
import java.util.ArrayList;

public class Plugin extends com.dMSN.Plugin.AbstractPluginInstance
{
	private final File settingsFile = new File("resources/plugins/dFileShare/Settings.txt");

	private File root;
	private String password;
	private ArrayList loggedInContacts;

	public boolean[] getEnabledButtons()
	{
		return new boolean[]
		{
			root == null, // start button
			root != null, // stop button
			true, 		  // info button
			false		  // settings button
		};
	}

	public String getPluginStatus()
	{
		if(loggedInContacts == null)
		{
			return READY;
		}
		else
		{
			return root != null ? STARTED : STOPPED;
		}
	}

    public String getPluginAuthor()
    {
        return "Danny";
    }

	public String getPluginName()
	{
		return "dFileShare";
	}

	public void stopPlugin()
	{
		loggedInContacts.clear();

		password = null;

		root = null;
	}

	public void startPlugin()
	{
		if(loggedInContacts == null)
			loggedInContacts = new ArrayList();

		try
		{
			BufferedReader br = new BufferedReader(new FileReader(settingsFile));
			String line = null;

			while ((line = br.readLine()) != null)
			{
				if(line.startsWith("SharedFolder"))
				{
					root = new File(line.substring(line.indexOf("=")+1).trim());
				}
				else if(line.startsWith("Password"))
				{
					password = line.substring(line.indexOf("=")+1).trim();
				}
			}

			br.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	public void showInfo(ActionEvent e)
	{
		JOptionPane.showMessageDialog(null, "This plugin allows you to act as a file server through IRC commands.\nThe commands are:\n!list to get the list of shared files.\n!get + filename or file number to get a filetransfer request.");
	}

    public void instantMessageReceived(String from, String to, String message) {

		// System.out.println(from + " said " + message + " to " + to);

		// user gebruikt goede password enzo
		if(password != null && message.equals("!login " + password))
		{
			loggedInContacts.add(from);
		}

		// User did not provide a password
		if(password != null && loggedInContacts.contains(from) == false && password.length() != 0)
		{
			return;
		}

		// nutteloze optie maja
		if(message.equalsIgnoreCase("!status"))
		{
			File[] list = root.listFiles();

			sendMessage(to, from, "Currently sharing " + list.length + " files.");

			return;
		}

		else if(message.equalsIgnoreCase("!list"))
		{
			File[] list = root.listFiles();

			if(list.length == 0)
			{
				sendMessage(to, from, "There are no files being shared at the moment.");
			}
			else
			{

				StringBuffer reply = new StringBuffer();

				for(int i=0;i<list.length;i++)
				{
					if(list[i].isFile())
					{
						reply.append((i+1) + ". " + list[i].getName() + "\n");
					}
				}

				sendMessage(to, from, reply.toString());
			}

			return;
		}

		else if(message.startsWith("!get "))
		{
			String fileName = message.substring(5);

			File[] list = root.listFiles();

			try
			{
				int index = Integer.parseInt(fileName);

				com.dMSN.Plugin.PluginRelay.sendFileOffer(to, from, list[index - 1]);

				return;
			}
			catch(Exception e)
			{
			}

			// zorgt ervoor dat alleen de files die in de lijst staan verstuurd mogen worden
			// dit voorkomt '../' acties waarvia een contact files uit een hogere directory kan opvragen
			for(int i=0;i<list.length;i++)
			{
				if(list[i].isFile() && list[i].getName().equals(fileName))
				{
					com.dMSN.Plugin.PluginRelay.sendFileOffer(to, from, new File(root, fileName));

					return;
				}
			}

			sendMessage(to, from, fileName + " not found");

			return;
		}
	}

	private void sendMessage(String from, String to, String message)
	{
		// ff een idiote work around voor die max van 1200 nogwat characters

		while(message != null)
		{
			if(message.length() > 1200)
			{
				com.dMSN.Plugin.PluginRelay.sendMessage(from, to, message.substring(0, 1200));
				message = message.substring(1200);
			}
			else
			{
				com.dMSN.Plugin.PluginRelay.sendMessage(from, to, message);
				message = null;
			}
		}
	}

}